%Code to calculate confidence set results of Andrews and Mikusheva
%(2014), "Maximum Likelihood Inference in Weakly Identified DSGE Models".
%For further information on options, see notes in "Test_comparison".


clear
global point_list loglike_list diff_precision  estim_params derivative_calc partial_list
global test_params calibrated_values AM_LM_output
global  fixJ Jsim AM_LM_J_fixed
global likelihood_eval point_test composite_test AM_LM Qu_LM
global simpoint J_MC_integration

%Note: This code is designed to work with Dynare (version 4.1.1), avalible at:
%http://www.dynare.org/
%and also uses commands from the e4 timeseries package for Matlab,
%availible at:
%http://www.ucm.es/info/icae/e4/

draws=10^4;

diff_precision=10^-6;
derivative_calc=1;

datalength=200;

likelihood_eval=0;
point_test=0;
composite_test=1;

AM_LM=1;
fixJ=0;
recalcJ=0;

Qu_LM=0;

GQIK=0;
restim=1;

DKK=1;
DKK_var_fix=1;
DKK_lags=3;


e4init
sete4opt('maxiter',200,'step',.2,'vcond','lyapunov');

J_MC_integration=0;

estim_params=ones(1,9);

test_params=[...
    1.0... %phi_x
    1.0... %phi_pie
    1.0... %alpha
    0.0... %rho
    0.0... %delta
    1.0... %kappa
    0.0... %sigma_a
    0.0... %sigma_u
    0.0... %sigma
    ];

label_vec={...
    '\phi_x';... %phi_x
    '\phi_{\pi}';... %phi_pie
    '\lambda';... %alpha
    '\rho';... %rho
    '\delta';... %delta
    '\kappa';... %kappa
    '\sigma_a';... %sigma_a
    '\sigma_u';... %sigma_u
    '\sigma'... %sigma
    };

beta=.99;
theta=.75;
phi=1;

true_value=[...
    2.2775... %phi_x
    2.0229... %phi_pie
    0.898... %alpha
    0.85... %rho
    0.1028... %delta
    0.1... %kappa
    0.3247... %sigma_a
    0.2649... %sigma_u
    0.5561... %sigma
    ];

Testrange=[0 4;...
    0, 4;...
    0, 0.99;...
    -0.99, 0.99;...
    -0.99, 0.99;...
    0, 3;...
    0, 4;...
    0, 4;...
    0, 4];

Jsim=0;
loglike_list=NaN;
partial_list=NaN;
calibrated_values=true_value;
point_list=calibrated_values;

tic1=tic;


AM_LM_store=NaN(draws,1);
AM_LM_fixedJ_store=NaN(draws,1);
paramval_store=NaN(draws,9);
count=1:9;
testparamscount=count(test_params==1);
simpoint=true_value;
Base_model_power_mod2

counter=0;
counter2=0;
while counter<draws
    counter2=counter2+1;
    rand('seed',counter2);
    calibrated_values=true_value;
    for k=1:sum(test_params)
        calibrated_values(testparamscount(k))=Testrange(testparamscount(k),1)...
            +rand*(Testrange(testparamscount(k),2)-Testrange(testparamscount(k),1));
    end
      
    simpoint=calibrated_values;
    xcheck=x;    
    Base_model_power_mod2
    counter/draws
    if mean(x==xcheck)<1 && rank([x r pie])==3
        Base_model_estimation_testeval_fix
    
    if AM_LM==1;
        AM_LM_store(counter+1)=AM_LM_output;
    end
    
    if fixJ==1;
        AM_LM_fixedJ_store(counter+1)=AM_LM_J_fixed;
    end
        
    paramval_store(counter+1,:)=calibrated_values;
    end  
    counter=sum(isnan(AM_LM_store)==0);
end

AM_CI=[min(paramval_store(AM_LM_store<chi2inv(.95,sum(test_params)),test_params==1))'...
    max(paramval_store(AM_LM_store<chi2inv(.95,sum(test_params)),test_params==1))'];


toc(tic1)